@@ -112,8 +112,8 @@ Options:
     - reads data from stdin.
     --interval generates timestamps at a regular interval into the past.'''),
     ('create',
-        '''Create a resource from a json description passed on stdin,
-    or using reasonable defaults.\n\nUsage:
+        '''Create a resource from a json description passed on stdin (with -),
+    or using command line shorthand (other variants).\n\nUsage:
     exo [options] create <cik> (--type=client|clone|dataport|datarule|dispatch) -
     exo [options] create <cik> --type=client
     exo [options] create <cik> --type=dataport (--format=binary|boolean|float|integer|string)
@@ -153,7 +153,7 @@ Options:
     --exclude=<key list>
                    comma separated list of info keys to include and exclude.
                    Available keys are aliases, basic, counts, description,
-                   key, shares, storage, subscribers, tags, usage. If omitted,
+                   key, shares, subscribers, tags, usage. If omitted,
                    all available keys are returned.'''),
     ('update',
         '''Update a resource from a json description passed on stdin.\n\nUsage:
@@ -209,7 +209,6 @@ Options:
     ('tree', '''Display a resource's descendants.\n\nUsage:
     exo [options] tree [--verbose] <cik>
 
-    --counts       show item counts (from info.storage.counts)
     --level=<num>  depth to traverse, omit or -1 for no limit [default: -1]'''),
     #('ut', '''Display a tree as fast as possible\n\nUsage:
     #exo [options] ut <cik>'''),
@@ -255,7 +254,10 @@ Options:
     If only --write arguments are specified, the call is a write.
     If only --read arguments are specified, the call is a read.
     If both --write and --read arguments are specified, the hybrid
-        write/read API is used. Writes are executed before reads.''')
+        write/read API is used. Writes are executed before reads.'''),
+    ('activate', '''Activate a model-backed device, based on vendor name,
+    vendor model and serial number.\n\nUsage:
+     exo [options] activate <vendor> <model> <sn>''')
     ])
 
 # shared sections of documentation
@@ -764,8 +766,6 @@ class ExoRPC():
                 ridopt = True
         add_opt(ridopt, 'rid', rid)
         add_opt('--verbose', 'unit', units)
-        if 'storage' in info and 'count' in info['storage']:
-            add_opt(True, 'count', info['storage']['count'])
 
         if 'format' in info['description']:
             desc = info['description']['format'] + ' ' + typ + ' ' + id
@@ -788,9 +788,6 @@ class ExoRPC():
         if isroot:
             # usage and counts are slow, so omit them if we don't need them
             exclude = ['usage']
-            if not cli_args['--counts']:
-                exclude.append('counts')
-                exclude.append('storage')
             info_options = self.make_info_options(exclude=exclude)
             #pprint(info_options)
             # todo: combine these (maybe also the _listing_with_info function
@@ -849,137 +846,6 @@ class ExoRPC():
                     else:
                         self._print_node(rid, info, aliases, cli_args, own_spacer, islast, max_name)
 
-    def _uberlistingtree(self, cik, rid=None, stats={'calls': 0}):
-        '''Get something like this: {'rid': '<rid1>',
-                                     'type': 'client',
-                                     'children': [{'rid': '<rid2>',
-                                                   'type': 'dataport'}]}'''
-        types = ['client', 'dataport', 'datarule', 'dispatch']
-        # result tree
-        tree = {}
-        # generation 0 (self)
-        if rid is None:
-            rid, listing = self._exomult(cik, [['lookup', 'aliased', ''],
-                                               ['listing', types]])
-            stats['calls'] += 1
-        else:
-            # connect_as() is an awkward API. What advantage does it have
-            # over passing in an auth dict?
-            self.exo.connect_as(rid)
-            listing = self._exomult(cik, [['listing', types]])[0]
-            stats['calls'] += 1
-            self.exo.connect_as(None)
-
-        tree['rid'] = rid
-
-        # children should look like this:
-        # [{'type': 'client',
-        #   'rid': '<rid1>'},
-        #  {'type': 'dataport',
-        #   'rid': '<rid2>'}]
-        def children(listing):
-            return [{'type': type, 'rid': rid} for i, type in enumerate(types) for rid in listing[i]]
-        tree['children'] = children(listing)
-
-        for node in tree['children']:
-            if node['type'] == 'client':
-                subtree = self._uberlistingtree(cik, node['rid'], stats)
-                if 'children' not in node:
-                    node['children'] = []
-                node['children'] = subtree['children']
-
-        return tree
-
-
-    def _uberinfotree(self, cik):
-        '''Get all info for a cik and its children in a nested dict.
-        The basic unit is {'rid': '<rid>',
-                           'info': <info>,
-                           'children': {'client': [child1],
-                                        'dataport': [child2]}
-                                        }
-        where <info-with-children> is just the info object for that node
-        with the addition of 'children' key, which is a dict containing
-        more nodes. Here's an example return value:
-
-           {'rid': '<rid 0>',
-            'info': {'description': ...,
-                     'basic': ....,
-                     ...},
-            'children: [{'rid': '<rid 1>',
-                         'info': {'description': ...,
-                                  'basic': ...},
-                         'children': [{'rid': '<rid 2>',
-                                       'info': {'description': ...,
-                                                'basic': ...},
-                                       'children: [] } } },
-                                      {'rid': '<rid 3>',
-                                       'info': {'description': ...,
-                                                'basic': ...},
-                                       'children': {} } }] } }
-
-           As it's building this nested dict, it calls nodeidfn with the rid and info
-           (w/o children) for each node.
-        '''
-        types = ['client', 'dataport', 'datarule', 'dispatch']
-
-        # result tree
-        tree = {}
-
-        # list of result nodes to fill out in the next generation
-        nextgen = []
-
-        # generation 0 (self)
-        rid, listing = self._exomult(cik, [['lookup', 'aliased', ''],
-                                           ['listing', types]])
-
-        tree['rid'] = rid
-
-        # children should look like this:
-        # [{'type': 'client',
-        #   'rid': '<rid1>'},
-        #  {'type': 'dataport',
-        #   'rid': '<rid2>'}]
-        def children(listing):
-            return [{'type': type, 'rid': rid} for i, type in enumerate(types) for rid in listing[i]]
-        tree['children'] = children(listing)
-
-        # calculate next generation list
-        # TODO: paging
-        nextgen = tree['children']
-
-        while len(nextgen) > 0:
-            print(len(nextgen))
-            curgen = nextgen
-            nextgen = []
-
-            cmds = []
-            for node in curgen:
-                if node['type'] == 'client':
-                    cmds.append(['listing', types])
-
-            if len(cmds) > 0:
-                result = self._exomult(cik, cmds)
-            else:
-                break
-
-            result_idx = 0
-            for i, node in enumerate(curgen):
-                # TODO: add results to members of curgen, and update nextgen
-                if node['type'] == 'client':
-                    r = result[result_idx]
-                    node['children'] = children(r)
-                    result_idx += 1
-                    nextgen.extend(node['children'])
-
-        return tree
-
-    def ubertree(self, cik):
-        #pprint(self._uberinfotree(cik)) #, options={'key': True, 'basic': True}))
-        stats = {'calls': 0}
-        pprint(self._uberlistingtree(cik, rid=None, stats=stats)) #, options={'key': True, 'basic': True}))
-        print('calls: {0}'.format(stats['calls']))
-
     def drop_all_children(self, cik):
         isok, listing = self.exo.listing(cik,
             types=['client', 'dataport', 'datarule', 'dispatch'])
@@ -1308,8 +1174,7 @@ class ExoRPC():
                   ['basic', 'status'],
                   ['basic', 'modified'],
                   ['basic', 'activity'],
-                  ['data'],
-                  ['storage']]
+                  ['data']]
 
         if nochildren:
             info1 = self.info(cik1)
@@ -1363,7 +1228,6 @@ class ExoRPC():
                                                         'description',
                                                         'key',
                                                         'shares',
-                                                        'storage',
                                                         'subscribers',
                                                         'tags',
                                                         'usage']
@@ -1400,8 +1264,7 @@ class ExoData():
         headers = {'X-Exosite-CIK': cik,
                    'Content-Type': 'application/x-www-form-urlencoded; charset=utf-8'}
         url = self.url + '/onep:v1/stack/alias'
-        data = '&'.join(['='.join(t) for t in alias_values])
-        r = requests.post(url, headers=headers, data=data)
+        r = requests.post(url, headers=headers, data=alias_values)
         self.raise_for_status(r)
         return r.text
 
@@ -1410,8 +1273,7 @@ class ExoData():
                    'Content-Type': 'application/x-www-form-urlencoded; charset=utf-8',
                    'Accept': 'application/x-www-form-urlencoded; charset=utf-8'}
         url = self.url + '/onep:v1/stack/alias?' + '&'.join(aliases)
-        data = '&'.join(['='.join(t) for t in alias_values])
-        r = requests.post(url, headers=headers, data=data)
+        r = requests.post(url, headers=headers, data=alias_values)
         self.raise_for_status(r)
         return r.text
 
@@ -1421,6 +1283,28 @@ class ExoData():
         return r.text
 
 
+class ExoProvision():
+    '''Implements the Provision API
+    https://github.com/exosite/api/tree/master/provision'''
+
+    def __init__(self, url='http://m2.exosite.com'):
+        self.url = url
+
+    def raise_for_status(self, r):
+        try:
+            r.raise_for_status()
+        except Exception as ex:
+            raise ExoException(str(ex))
+
+    def activate(self, vendor, model, serialnumber):
+        headers = {'Content-Type': 'application/x-www-form-urlencoded; charset=utf-8',
+                   'Accept': 'application/x-www-form-urlencoded; charset=utf-8'}
+        url = self.url + '/provision/activate'
+        data = {'vendor': vendor, 'model': model, 'sn': serialnumber}
+        r = requests.post(url, headers=headers, data=data)
+        self.raise_for_status(r)
+        return r.text
+
 def parse_ts(s):
     return parse_ts_tuple(parser.parse(s).timetuple())
 
@@ -1735,7 +1619,12 @@ def handle_args(cmd, args):
             raise ExoException('--https, --port, and --debughttp are not supported for ip and data commands.')
         ed = ExoData(url='http://' + args['--host'])
 
-    if '<cik>' in args:
+    if cmd in ['activate']:
+        if args['--https'] is True or args['--port'] is not None or args['--debughttp'] is True:
+            # TODO: support these
+            raise ExoException('--https, --port, and --debughttp are not supported for ip and data commands.')
+        ep = ExoProvision(url='http://' + args['--host'])
+    if '<cik>' in args and args['<cik>'] is not None:
         cik = args['<cik>']
         if type(cik) is list:
             cik = [er.lookup_shortcut(c) for c in cik]
@@ -1932,8 +1821,6 @@ def handle_args(cmd, args):
     # special commands
     elif cmd == 'tree':
         er.tree(cik, cli_args=args)
-    #elif cmd == 'ut':
-    #    er.ubertree(cik)
     elif cmd == 'script':
         # cik is a list of ciks
         er.upload_script(cik, args['<script-file>'],
@@ -1990,6 +1877,8 @@ def handle_args(cmd, args):
         elif len(writes) > 0:
             alias_values = get_alias_values(writes)
             ed.write(cik, alias_values)
+    elif cmd == 'activate':
+        pr(ep.activate(args['<vendor>'], args['<model>'], args['<sn>']))
     else:
         # search plugins
         handled = False
@@ -2099,6 +1988,7 @@ def cmd(argv=None, stdin=None, stdout=None, stderr=None):
     except KeyboardInterrupt:
         if args['--debug']:
             raise
+
     return 0
 
 
@@ -2109,24 +1999,33 @@ class CmdResult():
         self.stderr = stderr
 
 
-def run(argv, stdin):
+def run(argv, stdin=None):
     '''Runs an exoline command, translating stdin from
     string and stdout to string. Returns a CmdResult.'''
-    if type(stdin) is str:
-        sio = StringIO.StringIO()
-        sio.write(stdin)
-        sio.seek(0)
-        stdin = sio
-    stdout = StringIO.StringIO()
-    stderr = StringIO.StringIO()
+    old = {'stdin': sys.stdin, 'stdout': sys.stdout, 'stderr': sys.stderr}
+    try:
+        if stdin is None:
+            stdin = sys.stdin
+        elif type(stdin) is str:
+            sio = StringIO.StringIO()
+            sio.write(stdin)
+            sio.seek(0)
+            stdin = sio
+        stdout = StringIO.StringIO()
+        stderr = StringIO.StringIO()
 
-    # unicode causes problems in docopt
-    argv = [str(a) for a in argv]
-    exitcode = cmd(argv=argv, stdin=stdin, stdout=stdout, stderr=stderr)
-    stdout.seek(0)
-    stdout = stdout.read().strip()  # strip to get rid of leading newline
-    stderr.seek(0)
-    stderr = stderr.read().strip()
+        # unicode causes problems in docopt
+        argv = [str(a) for a in argv]
+        exitcode = cmd(argv=argv, stdin=stdin, stdout=stdout, stderr=stderr)
+        stdout.seek(0)
+        stdout = stdout.read().strip()  # strip to get rid of leading newline
+        stderr.seek(0)
+        stderr = stderr.read().strip()
+    finally:
+        # restore stdout, stderr, stdin
+        sys.stdin = old['stdin']
+        sys.stdout = old['stdout']
+        sys.stderr = old['stderr']
     return CmdResult(exitcode, stdout, stderr)
 
 
