@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 """Exoline test
    Tests exoline commands. Requires portalcik to be set in config.py.
 
@@ -12,6 +13,7 @@ from datetime import datetime
 import logging
 from unittest import TestCase
 import itertools
+import os
 
 from dateutil import parser
 
@@ -82,10 +84,10 @@ class TestRPC(TestCase):
     RE_RID = '[0-9a-f]{40}'
 
     def _logall(self, r):
-        self.l('stdout: {0}\nstdout length:{1}\nstderr: {2}\nstderr length:{3}'.format(abbrev(r.stdout),
-                                                                                       len(r.stdout),
-                                                                                       abbrev(r.stderr),
-                                                                                       len(r.stderr)))
+        self.l(u'stdout: {0}\nstdout length:{1}\nstderr: {2}\nstderr length:{3}'.format(abbrev(r.stdout),
+                                                                                        len(r.stdout),
+                                                                                        abbrev(r.stderr),
+                                                                                        len(r.stderr)))
 
     def _stdre(self, r, msg, search, match, stderr=False):
         std, label = (r.stderr, "stderr") if stderr else (r.stdout, "stdout")
@@ -178,7 +180,7 @@ class TestRPC(TestCase):
         self.assertEqual(r.exitcode, 0, 'create succeeds')
 
         rid = re.match('rid: ({0})'.format(self.RE_RID), r.stdout).groups()[0]
-        ri = rpc('info', res.parentcik, rid, '--exclude=count,usage')
+        ri = rpc('info', res.parentcik, rid, '--exclude=counts,usage')
         info = json.loads(ri.stdout.strip())
         res.created(rid, info)
 
@@ -263,6 +265,7 @@ Asked for desc: {0}\ngot desc: {1}'''.format(res.desc, res.info['description']))
                 res.rid,
                 '--limit={0}'.format(limit),
                 '--timeformat=unix')
+        log.debug(r.stdout)
         lines = r.stdout.split('\n')
         vread = []
         for line in lines:
@@ -846,7 +849,7 @@ Asked for desc: {0}\ngot desc: {1}'''.format(res.desc, res.info['description']))
     def info_test(self):
         '''Info command'''
         allkeys = ['aliases', 'basic', 'counts', 'description', 'key',
-                   'shares', 'storage', 'subscribers', 'tags', 'usage']
+                   'shares', 'subscribers', 'tags', 'usage']
         cik = self.client.parentcik
         rid = self.client.rid
 
@@ -1031,12 +1034,71 @@ Asked for desc: {0}\ngot desc: {1}'''.format(res.desc, res.info['description']))
     def spec_test(self):
         '''Spec command'''
         cik = self.client.cik()
-        spec = 'files/spec.yml'
+        spec = 'files/spec.yaml'
         r = rpc('spec', cik, spec)
         self.notok(r, 'no ids specified')
         r = rpc('spec', cik, spec, '--ids=1,2')
-        self.ok(r, 'specify ids, client doesn''t match', search='Failed to get info and data')
+        self.ok(r, 'specify ids, client doesn''t match', search='not found')
         r = rpc('spec', cik, spec, '--ids=1,2', '--create')
         self.ok(r, 'create dataports and scripts based on spec')
         r = rpc('spec', cik, spec, '--ids=1,2')
         self.ok(r, 'create dataports and scripts based on spec', match='')
+
+        # test correct number of children of each type
+        r = rpc('listing', cik, '--type=dataport', '--type=datarule')
+        self.ok(r, 'get listing of client')
+        dataports, datarules = json.loads(r.stdout)
+        self.assertEqual(len(dataports), 5, 'created correct number of dataports')
+        self.assertEqual(len(datarules), 3, 'created correct number of datarules')
+
+        # test initial value
+        r = rpc('read', cik, 'teststring_set1', '--format=raw')
+        self.ok(r, 'check that initial value was created', match='my initial value')
+
+        # test no initial value
+        r = rpc('read', cik, 'teststring', '--format=raw')
+        self.ok(r, 'check that no initial value was created', match='')
+
+        # test format
+        r = rpc('info', cik, 'teststring', '--include=description')
+        self.ok(r, 'got info for teststring')
+        description = json.loads(r.stdout)['description']
+        self.assertEqual(description['format'], 'string', 'teststring is a string')
+
+        # test format
+        r = rpc('info', cik, 'testint_units', '--include=description')
+        self.ok(r, 'got info for testint_units')
+        description = json.loads(r.stdout)['description']
+        self.assertEqual(description['format'], 'integer', 'testints_units is an integer')
+
+        # test unit
+        meta_string = description['meta']
+        self.assertFalse(len(meta_string) == 0)
+        meta = json.loads(meta_string)
+        self.assertEqual(meta['datasource']['unit'], u'°F', 'unit set correctly based on spec')
+
+        # test format content
+        r = rpc('read', cik, 'testjson', '--format=raw')
+        self.ok(r, 'read testjson initial value', match='{}')
+
+        # TODO: test lua script templating
+        # TODO: test that correct differences are reported. Change one thing
+        # at a time, confirm report is correct, then confirm passing --create
+        # resolves it.
+
+        # text --example script
+        r = rpc('drop', cik, '--all-children')
+        self.ok(r, 'drop children from test client')
+        r = rpc('spec', '--example')
+        example_spec = 'files/tmp_examplespec.yaml'
+        with open(example_spec, 'w') as f:
+            print(r.stdout)
+            f.write(r.stdout)
+        r = rpc('spec', cik, example_spec, '--ids=A,B')
+        self.ok(r, 'empty client does not match example spec', match='.+')
+        r = rpc('spec', cik, example_spec, '--create', '--ids=A,B')
+        self.ok(r, 'create from example spec')
+        r = rpc('spec', cik, example_spec, '--ids=A,B')
+        self.ok(r, 'client now matches example spec', match='')
+
+        os.remove(example_spec)
